.TH ilutp G "March 2000" "Scilab Group" "Scilab Function"
.so ../sci.an
.SH NAME
ilutp - sparse incomplete lu factorization with truncation and pivoting
.SH CALLING SEQUENCE
.nf
[L,U,perm,ierr]=ilutp(A,lfil,drop,ptol,bloc)
.fi
.SH PARAMETERS
.TP
A
: sparse matrix (must be square)
.TP
lfil
: the fill-in parameter; The default of \fVlfil\fR is the average number of nonnull elements of the matrix A by lines
.TP
drop
: the threshold for dropping small terms in the factorization; \fVdrop\fR uses the default, 0.001*max(Aij)
.TP
ptol
: tolerance ratio used to determne whether or not to permute two columns. \fVptol\fR uses the default, 0.5
.TP
bloc
: permuting can be done within the diagonal blocks of size mbloc. \fVbloc\fR uses the default n, with n the size of A
.TP
L
: lower triangular matrix
.TP
U
: upper triangular matrix
.TP
perm
: contains the permutation arrays
.TP
ierr
: error message
.SH DESCRIPTION
\fV[L,U,perm,ierr]=ilutp(A,lfil,drop,ptol,bloc)\fR builds an incomplete LU factorization of a sparse matrix \fVA\fR. The two factors L and U are stored in the CSR format. The CSR format is the Compressed Sparse Row format used by Scilab.
.TP 
lfil
: Each row of L and each row of U will have a maximum of \fVlfil\fR elements (excluding the diagonal element). lfil must be .ge. 0.
.TP
ptol
: tolerance ratio used to  determne whether or not to permute two columns. At step i columns i and j are permuted when abs(a(i,j))*\fVptol\fR .gt. abs(a(i,i)). [0 --> never permute; good values 0.1 to 0.01]
.TP
bloc
: Useful for PDE problems with several degrees of freedom.. If feature not wanted take \fVbloc\fR=n.
.TP
ierr
: Error message with the following meaning.
.LP
   0    --> successful return.
.LP
  >0  --> zero pivot encountered at step number ierr.
.LP
  -1   --> Error. input matrix may be wrong. (The elimination process has generated a row in L or U whose length is .gt.  n.)
.LP
  -2   --> The matrix L overflows the array al.
.LP
  -3   --> The matrix U overflows the array alu.
.LP
  -4   --> Illegal value for lfil.
.LP
  -5   --> zero row encountered.
.SH EXAMPLE
.nf
A=mmread(SCILIN+'/demos/matrices/pde225.mtx')
n=size(A,1); b=ones(n,1);
lfil=10;drop=0;ptol=0;bloc=3;
[L,U,perm,ierr]=ilutp(A,lfil,drop,ptol,bloc);
x=U\(L\b)
A*x-b
.fi
.SH AUTHOR
Aladin Group & Yousef Saad
.SH SEE ALSO
ilut, sparse