#include <malloc.h>
#include <math.h>
#include <stdio.h>
#include "stack-c.h"
#include "DoubleMFQR.h"

typedef short ftnlen;


struct FrontalMatrix
{
  long Lines,Columns;
  long *Lin,*Col;
  double *Val;
  struct FrontalMatrix *Next;
};

struct TempR
{
  long n;
  long *Col;
  double *Val;
};


static int ReduceNode(long CurrentNode,long FatherNode,
		      long *SRow,long *Col,double *Val,long dim,
		      struct FrontalMatrix **ContribFM,
		      struct TempR *R0,long *NzR)
{
  struct FrontalMatrix NodeFM,*CurrentContrib;
  long j,k,kend,l;
  long *CPattern;
  double *v,Tau,*Work;
  double Normv,Normv2;
  char L[]="L";
  long inc=1;
  extern double dnrm2_();
  int dlarfx_(char *side,
	      long *m,
	      long *n,
	      double *v,
	      double *tau,
	      double *c__,
	      long *ldc,
	      double *work,
	      ftnlen side_len);
  ftnlen side_len=1;
  long *p,*q;
  long *pi,*qi;
  
  NodeFM.Lines=NodeFM.Columns=0;
  if ((CPattern=(long *)malloc(dim*sizeof(long)))==NULL)
    {
      printf("MFQR: Out of memory\n");
      exit(-1);
    }
  memset(CPattern,0,dim*sizeof(long));
  /* Building Column Pattern of Frontal Matrix */
  NodeFM.Columns=0;
  for(j=0;j<dim;j++)
    if (Col[SRow[j]]==CurrentNode)
      {
        NodeFM.Lines++;
	p=&Col[SRow[j]];
	q=&Col[SRow[j+1]];
	while(p!=q)
	  {
	    if (CPattern[*p]==0)
	      {
		NodeFM.Columns++;
		CPattern[*p]=1;
	      }
	    p++;
	  }
      }
  /* Appending Pattern of contribution matrix */
  CurrentContrib=ContribFM[CurrentNode];
  while (CurrentContrib!=NULL)
    {
      NodeFM.Lines+=CurrentContrib->Lines;
      pi=CurrentContrib->Col;
      qi=&CurrentContrib->Col[CurrentContrib->Columns];
      while(pi!=qi)
	{
	  if (CPattern[*pi]==0)
	    {
	      NodeFM.Columns++;
	      CPattern[*pi]=1;
	    }
	  pi++;
	}
      CurrentContrib=CurrentContrib->Next;	  
    }
  if ((NodeFM.Lin=(long *)malloc(NodeFM.Lines*sizeof(long)))==NULL)
    {
      printf("MFQR: Out of memory\n");
      exit(-1);
    }
  if ((NodeFM.Col=(long *)malloc(NodeFM.Columns*sizeof(long)))==NULL)
    {
      printf("MFQR: Out of memory\n");
      exit(-1);
    }
  if ((NodeFM.Val=(double *)malloc(NodeFM.Lines*NodeFM.Columns*sizeof(double)))==NULL)
    {
      printf("MFQR: Out of memory\n");
      exit(-1);
    }
  memset(NodeFM.Val,0,NodeFM.Lines*NodeFM.Columns*sizeof(double));
  k=0;
  for(j=0;j<dim;j++)
    if (CPattern[j]==1)
      {
	CPattern[j]=k;
	NodeFM.Col[k]=j;
	k++;
      }
  l=0;
  for(j=0;j<dim;j++)
    if (Col[SRow[j]]==CurrentNode)
      {
	kend=SRow[j+1];
	NodeFM.Lin[l]=j;
        k=SRow[j];
        p=&Col[k];
        q=&Col[SRow[j+1]];
        while (p!=q)
	  NodeFM.Val[l+NodeFM.Lines*CPattern[*p++]]=Val[k++];
	l++;
      }
  while(ContribFM[CurrentNode]!=NULL)
    {
      CurrentContrib=ContribFM[CurrentNode];
      ContribFM[CurrentNode]=CurrentContrib->Next;
      for(j=0;j<CurrentContrib->Lines;j++)
	{
          NodeFM.Lin[l]=CurrentContrib->Lin[j];
          pi=CurrentContrib->Col;
          for(k=0;k<CurrentContrib->Columns;k++)
            NodeFM.Val[l+NodeFM.Lines*CPattern[*pi++]]=
	      CurrentContrib->Val[j+k*CurrentContrib->Lines];
          l++;
	}
      free(CurrentContrib->Lin);
      free(CurrentContrib->Col);
      free(CurrentContrib->Val);
      free(CurrentContrib);
    }
  free(CPattern);
  Work=(double *)malloc((NodeFM.Columns)*sizeof(double));
  v=(double *) malloc(NodeFM.Lines*sizeof(double));
  memcpy(v,NodeFM.Val,NodeFM.Lines*sizeof(double));
  Tau= v[0]<0?-1:1;
  Normv=dnrm2_(&NodeFM.Lines,v,&inc);
  Tau*=Normv;
  Normv2=2*v[0]*Tau+Tau*Tau;
  v[0]+=Tau;
  Normv=Normv*Normv+Normv2;
  Tau=2/Normv;
  dlarfx_(L,&NodeFM.Lines,&NodeFM.Columns,v,&Tau,NodeFM.Val,
	  &NodeFM.Lines,Work,side_len);
  free(v); 
  free(Work);
  R0[CurrentNode].n=NodeFM.Columns;
  R0[CurrentNode].Col=(long *)malloc(R0[CurrentNode].n*sizeof(long));
  memcpy(R0[CurrentNode].Col,NodeFM.Col,R0[CurrentNode].n*sizeof(long *));
  R0[CurrentNode].Val=(double*)malloc(R0[CurrentNode].n*sizeof(double));
  for(j=0;j<NodeFM.Columns;j++)
    R0[CurrentNode].Val[j]=NodeFM.Val[j*NodeFM.Lines];
  if (CurrentNode<dim-1 && NodeFM.Lines>1)
    {
     CurrentContrib=ContribFM[FatherNode];
     ContribFM[FatherNode]=(struct FrontalMatrix *)malloc(sizeof(struct FrontalMatrix));
     if (ContribFM[FatherNode]==NULL)
       {
	 printf("MFQR: Out of memory\n");
	 exit(-1);
       }
     ContribFM[FatherNode]->Next=CurrentContrib;
     CurrentContrib=ContribFM[FatherNode];
     CurrentContrib->Lines=NodeFM.Lines-1;
     CurrentContrib->Columns=NodeFM.Columns-1;
     if ((CurrentContrib->Lin=(long *)malloc(CurrentContrib->Lines*sizeof(long)))==NULL)
       {
	 printf("MFQR: Out of memory\n");
	 exit(-1);
       }
     if ((CurrentContrib->Col=(long *)malloc(CurrentContrib->Columns*sizeof(long)))==NULL)
       {
	 printf("MFQR: Out of memory\n");
	 exit(-1);
       }
     if ((CurrentContrib->Val=
	  (double *)malloc(CurrentContrib->Lines*CurrentContrib->Columns*sizeof(double)))==NULL)
       {
	 printf("MFQR: Out of memory\n");
	 exit(-1);
       }
     memcpy(CurrentContrib->Lin,&NodeFM.Lin[1],CurrentContrib->Lines*sizeof(long));
     memcpy(CurrentContrib->Col,&NodeFM.Col[1],CurrentContrib->Columns*sizeof(long));
     for(j=0;j<CurrentContrib->Columns;j++)
       memcpy(&CurrentContrib->Val[j*CurrentContrib->Lines],
	      &NodeFM.Val[1+(j+1)*NodeFM.Lines],
	      CurrentContrib->Lines*sizeof(double));
    }
  free(NodeFM.Lin);
  free(NodeFM.Col);
  free(NodeFM.Val);
  (*NzR)+=R0[CurrentNode].n;
  return 1;
} 
 
 
static int ReduceNodes(long *Pere,long Node,long *SRow,long *Col,double *Val,long dim,
			struct FrontalMatrix **ContribFM,
			struct TempR *R0,long *NzR)
{
  long i;
  for(i=0;i<Node;i++)
    if (Pere[i]==Node)
      ReduceNodes(Pere,i,SRow,Col,Val,dim,ContribFM,R0,NzR);
  return ReduceNode(Node,Pere[Node],SRow,Col,Val,dim,ContribFM,R0,NzR);
}


struct List
{
  long Node;
  struct List *Next;
};

static struct List *AddList(int Node,struct List *l,char *FillIn)
{
  struct List *p,*q;

  p=l;
  while (p && p->Node<Node)
    {
      q=p;
      p=p->Next;
    }
  if (p==l && (p==NULL || p->Node>Node))
    {
      l=(struct List *)malloc(sizeof(struct List));
      l->Next=p;
      l->Node=Node;
      *FillIn = 'F';
      return l;
    }
  if (p==NULL || p->Node>Node)
    {
      q->Next=(struct List *)malloc(sizeof(struct List));
      q->Next->Next=p;
      q->Next->Node=Node;
      *FillIn = 'F';
      return l;
    }
  *FillIn = 'N';
  return l;
}
 

static long *EliminationTree(long *SRow,long *Col,long n)
     /* n=Nombre de lignes de la matrice 

          SRow: long SRow[n+1]
	     vecteur qui indique le debut de chaque ligne
          Col: long Col[Nnz]
     */
{
  struct List **AdjGraph,*tmp;
  register long i,j,k;
  long *Pere,Fillin=0;
  char Status;

  AdjGraph=(struct List **)malloc(n*sizeof(struct List *));
  Pere=(long *)malloc(n*sizeof(long));
  for(i=0;i<n;i++)
    {
      AdjGraph[i]=NULL;
    }
  /* Construire le graphe de A'*A */
  for(i=0;i<n;i++)
    {
      for(j=SRow[i];j<SRow[i+1];j++)
	{
	  for(k=j+1;k<SRow[i+1];k++)
	    {
	      AdjGraph[Col[j]]=AddList(Col[k],AdjGraph[Col[j]],&Status);
	      if (Status=='F')
		{
		  Fillin++;
		  AdjGraph[Col[k]]=AddList(Col[j],AdjGraph[Col[k]],&Status);
		}
	    }
	}
    }
  /* Construire le graphe de dependance */
  for (i=0;i<n;i++)
    {
      /* Trouver le pere du noeud i */
      Pere[i]=n-1;
      if (AdjGraph[i])
	Pere[i]=AdjGraph[i]->Node;
      /* Eliminer le noeud i du graphe en ajustant les connexions */
      while (AdjGraph[i])
	{
	  /* Eliminer le premier Element dans la liste d'adjacence du noeud
                     AdjGraph[AdjGraph[i]
		     puisque c'est le plus petit et il correspond a i */
	  tmp=AdjGraph[AdjGraph[i]->Node];
	  AdjGraph[AdjGraph[i]->Node]=tmp->Next;
	  free(tmp);
          /* Mettre a jour les connexions du graphe apres l'elimination du noeud i */
	  tmp=AdjGraph[i]->Next;
	  while (tmp)
	    {
	      if (tmp->Node>i)
		{
		  AdjGraph[tmp->Node]=
		    AddList(AdjGraph[i]->Node,AdjGraph[tmp->Node],&Status);
		  if (Status=='F')
		    {
		      Fillin++;
		      AdjGraph[AdjGraph[i]->Node]=
			AddList(tmp->Node,AdjGraph[AdjGraph[i]->Node],&Status);
		    }
		}
	      tmp=tmp->Next;
	    }
	  /* Detruire la liste du Noeud i */
	  tmp=AdjGraph[i];
	  AdjGraph[i]=tmp->Next;
	  free(tmp);
	}
    }
  /* Liberer la memoire allouee et sortir */
  while(AdjGraph[n-1])
    {
      tmp=AdjGraph[n-1];
      AdjGraph[n-1]=tmp->Next;
      free(tmp);
    }
  free(AdjGraph);
  return Pere;
}


/****************************************************/

struct DMatrix *dmfqr(long *SRow,long *Col,double *Val,long dim)
{
  long *Pere,i,k,NzR=0;
  struct FrontalMatrix **ContribFM;
  struct DMatrix *R;
  struct TempR *R0;
  
  Pere=EliminationTree(SRow,Col,dim);

  if ((ContribFM=
       (struct FrontalMatrix **)malloc(dim*sizeof(struct FrontalMatrix *)))==NULL)
    {
      free(Pere);
      return NULL;
    }
  memset(ContribFM,0,dim*sizeof(struct FrontalMatrix *));
  if ((R0=(struct TempR *)malloc(dim*sizeof(struct TempR)))==NULL)
    {
      free(Pere);
      free(ContribFM);
      return NULL;
    }
  memset(R0,0,dim*sizeof(struct TempR));
  ReduceNodes(Pere,dim-1,SRow,Col,Val,dim,ContribFM,R0,&NzR);
  free(ContribFM);
  free(Pere);
  if ((R=(struct DMatrix *)malloc(sizeof(struct DMatrix)))==NULL)
    {
      printf("MFQR: Out of memory\n");
      exit(-1);
    }
  R->dim=dim;
  if ((R->Val=(double *)malloc(NzR*sizeof(double)))==NULL)
    {
      printf("MFQR: Out of memory\n");
      exit(-1);
    }
  if ((R->Col=(long *)malloc(NzR*sizeof(long)))==NULL)
    {
      printf("MFQR: Out of memory\n");
      exit(-1);
    }
  if ((R->SRow=(long *)malloc((dim+1)*sizeof(long)))==NULL)
    {
      printf("MFQR: Out of memory\n");
      exit(-1);
    }
  k=0;
  R->SRow[0]=0;
  for(i=0;i<dim;i++)
    {
      memcpy(&R->Val[k],R0[i].Val,R0[i].n*sizeof(double));  
      memcpy(&R->Col[k],R0[i].Col,R0[i].n*sizeof(long));  
      k+=R0[i].n;
      R->SRow[i+1]=k;  
      free(R0[i].Val);
      free(R0[i].Col);
    }  
  free(R0);
  return R;
}
